@@ -4,6 +4,8 @@ from dynaconf.base import LazySettings  # noqa
 from dynaconf.constants import DEFAULT_SETTINGS_FILES
 from dynaconf.contrib import DjangoDynaconf  # noqa
 from dynaconf.contrib import FlaskDynaconf  # noqa
+from dynaconf.utils.inspect import inspect_settings
+from dynaconf.utils.parse_conf import add_converter  # noqa
 from dynaconf.validator import ValidationError  # noqa
 from dynaconf.validator import Validator  # noqa
 
@@ -18,6 +20,7 @@ settings = LazySettings(
     default_settings_paths=DEFAULT_SETTINGS_FILES,
 )
 
+
 # This is the new recommended base class alias
 Dynaconf = LazySettings  # noqa
 
@@ -28,4 +31,6 @@ __all__ = [
     "FlaskDynaconf",
     "ValidationError",
     "DjangoDynaconf",
+    "add_converter",
+    "inspect_settings",
 ]
