@@ -10,15 +10,19 @@ from collections import defaultdict
 from contextlib import contextmanager
 from contextlib import suppress
 from pathlib import Path
+from typing import Any
+from typing import Callable
 
 from dynaconf import default_settings
 from dynaconf.loaders import default_loader
 from dynaconf.loaders import enable_external_loaders
 from dynaconf.loaders import env_loader
-from dynaconf.loaders import execute_hooks
+from dynaconf.loaders import execute_instance_hooks
+from dynaconf.loaders import execute_module_hooks
 from dynaconf.loaders import py_loader
 from dynaconf.loaders import settings_loader
 from dynaconf.loaders import yaml_loader
+from dynaconf.loaders.base import SourceMetadata
 from dynaconf.utils import BANNER
 from dynaconf.utils import compat_kwargs
 from dynaconf.utils import ensure_a_list
@@ -31,10 +35,12 @@ from dynaconf.utils.boxing import DynaBox
 from dynaconf.utils.files import find_file
 from dynaconf.utils.functional import empty
 from dynaconf.utils.functional import LazyObject
+from dynaconf.utils.parse_conf import apply_converter
 from dynaconf.utils.parse_conf import converters
-from dynaconf.utils.parse_conf import get_converter
+from dynaconf.utils.parse_conf import Lazy
 from dynaconf.utils.parse_conf import parse_conf_data
 from dynaconf.utils.parse_conf import true_values
+from dynaconf.validator import ValidationError
 from dynaconf.validator import ValidatorList
 from dynaconf.vendor.box.box_list import BoxList
 
@@ -220,7 +226,7 @@ class Settings:
         self._deleted = set()
         self._store = DynaBox(box_settings=self)
         self._env_cache = {}
-        self._loaded_by_loaders = {}
+        self._loaded_by_loaders: dict[SourceMetadata, Any] = {}
         self._loaders = []
         self._defaults = DynaBox(box_settings=self)
         self.environ = os.environ
@@ -236,6 +242,9 @@ class Settings:
         self.validators = ValidatorList(
             self, validators=kwargs.pop("validators", None)
         )
+        self._post_hooks: list[Callable] = ensure_a_list(
+            kwargs.get("post_hooks", [])
+        )
 
         compat_kwargs(kwargs)
         if settings_module:
@@ -293,7 +302,7 @@ class Settings:
                     is False
                 ):
                     # only matches exact casing, first levels always upper
-                    return self._store.to_dict()[name]
+                    return self._store.__getattribute__(name)
                 # perform lookups for upper, and casefold
                 return self._store[name]
         # in case of RESERVED_ATTRS or KeyError above, keep default behaviour
@@ -339,10 +348,13 @@ class Settings:
         """Redirects to store object"""
         return self.store.values()
 
-    def setdefault(self, item, default, apply_default_on_none=False):
+    def setdefault(
+        self, item, default, apply_default_on_none=False, env: str = "unknown"
+    ):
         """Returns value if exists or set it as the given default
 
         apply_default_on_none: if True, default is set when value is None
+        env: used to create the source identifier
         """
         value = self.get(item, empty)
 
@@ -360,12 +372,15 @@ class Settings:
                 )
             )
         )
+        loader_identifier = SourceMetadata(
+            "validation_default", "unique", env.lower()
+        )
 
         if apply_default:
             self.set(
                 item,
                 default,
-                loader_identifier="setdefault",
+                loader_identifier=loader_identifier,
                 tomlfy=True,
             )
             return default
@@ -405,7 +420,7 @@ class Settings:
         # If we've reached the end, or parent key not found, then return result
         if not keys or result == default:
             if cast and cast in converters:
-                return get_converter(cast, result, box_settings=self)
+                return apply_converter(cast, result, box_settings=self)
             elif cast is True:
                 return parse_conf_data(result, tomlfy=True, box_settings=self)
             return result
@@ -423,6 +438,7 @@ class Settings:
         fresh=False,
         dotted_lookup=empty,
         parent=None,
+        sysenv_fallback=None,
     ):
         """
         Get a value from settings store, this is the preferred way to access::
@@ -436,8 +452,12 @@ class Settings:
         :param fresh: Should reload from loaders store before access?
         :param dotted_lookup: Should perform dotted-path lookup?
         :param parent: Is there a pre-loaded parent in a nested data?
+        :param sysenv_fallback: Should fallback to system environ if not found?
         :return: The value if found, default or None
         """
+        if sysenv_fallback is None:
+            sysenv_fallback = self._store.get("SYSENV_FALLBACK_FOR_DYNACONF")
+
         nested_sep = self._store.get("NESTED_SEPARATOR_FOR_DYNACONF")
         if nested_sep and nested_sep in key:
             # turn FOO__bar__ZAZ in `FOO.bar.ZAZ`
@@ -455,14 +475,23 @@ class Settings:
                 parent=parent,
             )
 
+        key = upperfy(key)
+
+        # handles system environment fallback
+        if default is None:
+            key_in_sysenv_fallback_list = isinstance(
+                sysenv_fallback, list
+            ) and key in [upperfy(k) for k in sysenv_fallback]
+            if sysenv_fallback is True or key_in_sysenv_fallback_list:
+                default = self.environ.get(key)
+
+        # default values should behave exactly Dynaconf parsed values
         if default is not None:
-            # default values should behave exactly Dynaconf parsed values
             if isinstance(default, list):
                 default = BoxList(default)
             elif isinstance(default, dict):
                 default = DynaBox(default)
 
-        key = upperfy(key)
         if key in self._deleted:
             return default
 
@@ -476,7 +505,7 @@ class Settings:
 
         data = (parent or self.store).get(key, default)
         if cast:
-            data = get_converter(cast, data, box_settings=self)
+            data = apply_converter(cast, data, box_settings=self)
         return data
 
     def exists(self, key, fresh=False):
@@ -515,7 +544,7 @@ class Settings:
         data = self.environ.get(key, default)
         if data:
             if cast in converters:
-                data = get_converter(cast, data, box_settings=self)
+                data = apply_converter(cast, data, box_settings=self)
             elif cast is True:
                 data = parse_conf_data(data, tomlfy=True, box_settings=self)
         return data
@@ -558,6 +587,7 @@ class Settings:
     @property
     def loaded_by_loaders(self):
         """Gets the internal mapping of LOADER -> values"""
+        # return {k.loader:data for k, data in self._loaded_by_loaders}
         return self._loaded_by_loaders
 
     def from_env(self, env="", keep=False, **kwargs):
@@ -622,6 +652,10 @@ class Settings:
         new_data["FORCE_ENV_FOR_DYNACONF"] = env
         new_settings = LazySettings(**new_data)
         self._env_cache[cache_key] = new_settings
+
+        # update source metadata for inspecting
+        self._loaded_by_loaders.update(new_settings._loaded_by_loaders)
+
         return new_settings
 
     @contextmanager
@@ -802,7 +836,9 @@ class Settings:
         for key in keys:
             self.unset(key, force=force)
 
-    def _dotted_set(self, dotted_key, value, tomlfy=False, **kwargs):
+    def _dotted_set(
+        self, dotted_key, value, tomlfy=False, validate=empty, **kwargs
+    ):
         """Sets dotted keys as nested dictionaries.
 
         Dotted set will always reassign the value, to merge use `@merge` token
@@ -813,7 +849,12 @@ class Settings:
 
         Keyword Arguments:
             tomlfy {bool} -- Perform toml parsing (default: {False})
+            validate {bool} --
         """
+        if validate is empty:
+            validate = self.get(
+                "VALIDATE_ON_UPDATE_FOR_DYNACONF"
+            )  # pragma: nocover
 
         split_keys = dotted_key.split(".")
         existing_data = self.get(split_keys[0], {})
@@ -826,22 +867,26 @@ class Settings:
         tree[split_keys[-1]] = value
 
         if existing_data:
+            old_data = DynaBox(
+                {split_keys[0]: existing_data}, box_settings=self
+            )
             new_data = object_merge(
-                old=DynaBox({split_keys[0]: existing_data}),
+                old=old_data,
                 new=new_data,
                 full_path=split_keys,
             )
-        self.update(data=new_data, tomlfy=tomlfy, **kwargs)
+        self.update(data=new_data, tomlfy=tomlfy, validate=validate, **kwargs)
 
     def set(
         self,
         key,
         value,
-        loader_identifier=None,
+        loader_identifier: SourceMetadata | None = None,
         tomlfy=False,
         dotted_lookup=empty,
         is_secret="DeprecatedArgument",  # noqa
-        merge=False,
+        validate=empty,
+        merge=empty,
     ):
         """Set a value storing references for the loader
 
@@ -850,7 +895,11 @@ class Settings:
         :param loader_identifier: Optional loader name e.g: toml, yaml etc.
         :param tomlfy: Bool define if value is parsed by toml (defaults False)
         :param merge: Bool define if existing nested data will be merged.
+        :param validate: Bool define if validation will be triggered
         """
+        if validate is empty:
+            validate = self.get("VALIDATE_ON_UPDATE_FOR_DYNACONF")
+
         if dotted_lookup is empty:
             dotted_lookup = self.get("DOTTED_LOOKUP_FOR_DYNACONF")
 
@@ -861,12 +910,29 @@ class Settings:
 
         if "." in key and dotted_lookup is True:
             return self._dotted_set(
-                key, value, loader_identifier=loader_identifier, tomlfy=tomlfy
+                key,
+                value,
+                loader_identifier=loader_identifier,
+                tomlfy=tomlfy,
+                validate=validate,
             )
 
+        # Fix for #905
+        # parsed_conf default value was causing duplication
+        value_not_parsed = (
+            value
+            if loader_identifier
+            and loader_identifier.loader == "validation_default"
+            else None
+        )
+
         value = parse_conf_data(value, tomlfy=tomlfy, box_settings=self)
         key = upperfy(key.strip())
-        existing = getattr(self, key, None)
+
+        # Fix for #869 - The call to getattr trigger early evaluation
+        existing = (
+            getattr(self, key, None) if not isinstance(value, Lazy) else None
+        )
 
         if getattr(value, "_dynaconf_del", None):
             # just in case someone use a `@del` in a first level var.
@@ -880,6 +946,12 @@ class Settings:
         if getattr(value, "_dynaconf_merge_unique", False):
             # just in case someone use a `@merge_unique` in a first level var
             if existing:
+                # update SourceMetadata (for inspecting purposes)
+                loader_identifier = (
+                    loader_identifier._replace(merged=True)
+                    if loader_identifier
+                    else None
+                )
                 value = object_merge(existing, value.unwrap(), unique=True)
             else:
                 value = value.unwrap()
@@ -887,20 +959,41 @@ class Settings:
         if getattr(value, "_dynaconf_merge", False):
             # just in case someone use a `@merge` in a first level var
             if existing:
+                # update SourceMetadata (for inspecting purposes)
+                loader_identifier = (
+                    loader_identifier._replace(merged=True)
+                    if loader_identifier
+                    else None
+                )
                 value = object_merge(existing, value.unwrap())
             else:
                 value = value.unwrap()
 
         if existing is not None and existing != value:
             # `dynaconf_merge` used in file root `merge=True`
-            if merge:
+            if merge and merge is not empty:
+                loader_identifier = (
+                    loader_identifier._replace(merged=True)
+                    if loader_identifier
+                    else None
+                )
                 value = object_merge(existing, value)
             else:
-                # `dynaconf_merge` may be used within the key structure
-                # Or merge_enabled is set to True
-                value = self._merge_before_set(existing, value)
+                # Fix for #905
+                if (
+                    loader_identifier
+                    and loader_identifier.loader == "validation_default"
+                ):
+                    value = value_not_parsed
+                else:
+                    # `dynaconf_merge` may be used within the key structure
+                    # Or merge_enabled is set to True
+                    value, updated_identifier = self._merge_before_set(
+                        existing, value, loader_identifier, context_merge=merge
+                    )
+                    loader_identifier = updated_identifier
 
-        if isinstance(value, dict):
+        if isinstance(value, dict) and not isinstance(value, DynaBox):
             value = DynaBox(value, box_settings=self)
 
         self.store[key] = value
@@ -908,23 +1001,27 @@ class Settings:
         super().__setattr__(key, value)
 
         # set loader identifiers so cleaners know which keys to clean
-        if loader_identifier and loader_identifier in self.loaded_by_loaders:
-            self.loaded_by_loaders[loader_identifier][key] = value
+        if loader_identifier and loader_identifier in self._loaded_by_loaders:
+            self._loaded_by_loaders[loader_identifier][key] = value
         elif loader_identifier:
-            self.loaded_by_loaders[loader_identifier] = {key: value}
+            self._loaded_by_loaders[loader_identifier] = {key: value}
         elif loader_identifier is None:
             # if .set is called without loader identifier it becomes
             # a default value and goes away only when explicitly unset
             self._defaults[key] = value
 
+        if validate is True:
+            self.validators.validate()
+
     def update(
         self,
         data=None,
         loader_identifier=None,
         tomlfy=False,
-        merge=False,
+        merge=empty,
         is_secret="DeprecatedArgument",  # noqa
         dotted_lookup=empty,
+        validate=empty,
         **kwargs,
     ):
         """
@@ -943,24 +1040,50 @@ class Settings:
         :param loader_identifier: Only to be used by custom loaders
         :param tomlfy: Bool define if value is parsed by toml (defaults False)
         :param merge: Bool define if existing nested data will be merged.
+        :param validate: Bool define if validators will trigger automatically
         :param kwargs: extra values to update
         :return: None
         """
+
+        if validate is empty:
+            validate = self.get("VALIDATE_ON_UPDATE_FOR_DYNACONF")
+
         data = data or {}
         data.update(kwargs)
         for key, value in data.items():
-            self.set(
-                key,
-                value,
-                loader_identifier=loader_identifier,
-                tomlfy=tomlfy,
-                merge=merge,
-                dotted_lookup=dotted_lookup,
-            )
+            # update() will handle validation later
+            with suppress(ValidationError):
+                self.set(
+                    key,
+                    value,
+                    loader_identifier=loader_identifier,
+                    tomlfy=tomlfy,
+                    merge=merge,
+                    dotted_lookup=dotted_lookup,
+                    validate=validate,
+                )
+
+        # handle param `validate`
+        if validate is True:
+            self.validators.validate()
+        elif validate == "all":
+            self.validators.validate_all()
+
+    def _merge_before_set(
+        self,
+        existing,
+        value,
+        identifier: SourceMetadata | None = None,
+        context_merge=empty,
+    ):
+        """
+        Merge the new value being set with the existing value before set
+        Returns the merged value and the updated identifier (for inspecting).
+        """
+        # context_merge may come from file_scope or env_scope
+        if context_merge is empty:
+            context_merge = self.get("MERGE_ENABLED_FOR_DYNACONF")
 
-    def _merge_before_set(self, existing, value):
-        """Merge the new value being set with the existing value before set"""
-        global_merge = getattr(self, "MERGE_ENABLED_FOR_DYNACONF", False)
         if isinstance(value, dict):
             local_merge = value.pop(
                 "dynaconf_merge", value.pop("dynaconf_merge_unique", None)
@@ -969,24 +1092,30 @@ class Settings:
                 # In case `dynaconf_merge:` holds value not boolean - ref #241
                 value = local_merge
 
-            if global_merge or local_merge:
+            if local_merge or (context_merge and local_merge is not False):
+                identifier = (
+                    identifier._replace(merged=True) if identifier else None
+                )
                 value = object_merge(existing, value)
 
         if isinstance(value, (list, tuple)):
-            local_merge = (
-                "dynaconf_merge" in value or "dynaconf_merge_unique" in value
-            )
-            if global_merge or local_merge:
-                value = list(value)
-                unique = False
-                if local_merge:
-                    try:
-                        value.remove("dynaconf_merge")
-                    except ValueError:  # EAFP
-                        value.remove("dynaconf_merge_unique")
-                        unique = True
+            value = list(value)
+            local_merge = None
+            unique = False
+            if "dynaconf_merge" in value:
+                value.remove("dynaconf_merge")
+                local_merge = True
+            elif "dynaconf_merge_unique" in value:
+                value.remove("dynaconf_merge_unique")
+                local_merge = True
+                unique = True
+
+            if local_merge or (context_merge and local_merge is not False):
+                identifier = (
+                    identifier._replace(merged=True) if identifier else None
+                )
                 value = object_merge(existing, value, unique=unique)
-        return value
+        return value, identifier
 
     @property
     def loaders(self):  # pragma: no cover
@@ -1002,6 +1131,7 @@ class Settings:
     def reload(self, env=None, silent=None):  # pragma: no cover
         """Clean end Execute all loaders"""
         self.clean()
+        self._loaded_hooks.clear()
         self.execute_loaders(env, silent)
 
     def execute_loaders(
@@ -1030,12 +1160,18 @@ class Settings:
             enable_external_loaders(self)
 
             loaders = self.loaders
-
+        # non setting_file or py_module loaders
         for core_loader in loaders:
             core_loader.load(self, env, silent=silent, key=key)
 
         self.load_includes(env, silent=silent, key=key)
-        execute_hooks("post", self, env, silent=silent, key=key)
+        self._store._box_config["_bypass_evaluation"] = True
+
+        # execute hooks
+        execute_module_hooks("post", self, env, silent=silent, key=key)
+        execute_instance_hooks(self, "post", self._post_hooks)
+
+        self._store._box_config["_bypass_evaluation"] = False
 
     def pre_load(self, env, silent, key):
         """Do we have any file to pre-load before main settings file?"""
@@ -1045,7 +1181,7 @@ class Settings:
 
     def load_includes(self, env, silent, key):
         """Do we have any nested includes we need to process?"""
-        includes = self.get("DYNACONF_INCLUDE", [])
+        includes = ensure_a_list(self.get("DYNACONF_INCLUDE"))
         includes.extend(ensure_a_list(self.get("INCLUDES_FOR_DYNACONF")))
         if includes:
             self.load_file(path=includes, env=env, silent=silent, key=key)
@@ -1054,26 +1190,39 @@ class Settings:
             if last_loader and last_loader == env_loader:
                 last_loader.load(self, env, silent, key)
 
-    def load_file(self, path=None, env=None, silent=True, key=None):
+    def load_file(
+        self, path=None, env=None, silent=True, key=None, validate=empty
+    ):
         """Programmatically load files from ``path``.
 
+        When using relative paths, the basedir fallbacks in this order:
+        - ROOT_PATH_FOR_DYNACONF
+        - Directory of the last loaded file
+        - CWD
+
         :param path: A single filename or a file list
         :param env: Which env to load from file (default current_env)
         :param silent: Should raise errors?
         :param key: Load a single key?
+        :param validate: Should trigger validation?
         """
+        if validate is empty:
+            validate = self.get("VALIDATE_ON_UPDATE_FOR_DYNACONF")
+
         env = (env or self.current_env).upper()
         files = ensure_a_list(path)
         if files:
             already_loaded = set()
             for _filename in files:
 
-                if py_loader.try_to_load_from_py_module_name(
-                    obj=self, name=_filename, silent=True
-                ):
-                    # if it was possible to load from module name
-                    # continue the loop.
-                    continue
+                # load_file() will handle validation later
+                with suppress(ValidationError):
+                    if py_loader.try_to_load_from_py_module_name(
+                        obj=self, name=_filename, silent=True
+                    ):
+                        # if it was possible to load from module name
+                        # continue the loop.
+                        continue
 
                 root_dir = str(self._root_path or os.getcwd())
 
@@ -1099,14 +1248,24 @@ class Settings:
                 for path in paths + local_paths:
                     if path in already_loaded:  # pragma: no cover
                         continue
-                    settings_loader(
-                        obj=self,
-                        env=env,
-                        silent=silent,
-                        key=key,
-                        filename=path,
-                    )
-                    already_loaded.add(path)
+
+                    # load_file() will handle validation later
+                    with suppress(ValidationError):
+                        settings_loader(
+                            obj=self,
+                            env=env,
+                            silent=silent,
+                            key=key,
+                            filename=path,
+                            validate=validate,
+                        )
+                        already_loaded.add(path)
+
+        # handle param `validate`
+        if validate is True:
+            self.validators.validate()
+        elif validate == "all":
+            self.validators.validate_all()
 
     @property
     def _root_path(self):
@@ -1281,5 +1440,6 @@ RESERVED_ATTRS = (
         "_validate_only",
         "_validate_exclude",
         "_validate_only_current_env",
+        "_post_hooks",
     ]
 )
