@@ -9,6 +9,7 @@ import warnings
 import webbrowser
 from contextlib import suppress
 from pathlib import Path
+from typing import TYPE_CHECKING
 
 from dynaconf import constants
 from dynaconf import default_settings
@@ -27,13 +28,16 @@ from dynaconf.vendor import click
 from dynaconf.vendor import toml
 from dynaconf.vendor import tomllib
 
+
+if TYPE_CHECKING:  # pragma: no cover
+    from dynaconf.base import Settings
+
 os.environ["PYTHONIOENCODING"] = "utf-8"
 
 CWD = None
-try:
+with suppress(FileNotFoundError):
     CWD = Path.cwd()
-except FileNotFoundError:
-    pass
+
 EXTS = ["ini", "toml", "yaml", "json", "py", "env"]
 WRITERS = ["ini", "toml", "yaml", "json", "py", "redis", "vault", "env"]
 
@@ -42,7 +46,6 @@ ENC = default_settings.ENCODING_FOR_DYNACONF
 
 def set_settings(ctx, instance=None):
     """Pick correct settings instance and set it to a global variable."""
-
     global settings
 
     settings = None
@@ -63,23 +66,30 @@ def set_settings(ctx, instance=None):
 
             flask_app = ScriptInfo().load_app()
             settings = FlaskDynaconf(flask_app, **flask_app.config).settings
-            _echo_enabled and click.echo(
-                click.style(
-                    "Flask app detected", fg="white", bg="bright_black"
+            if _echo_enabled:
+                click.echo(
+                    click.style(
+                        "Flask app detected", fg="white", bg="bright_black"
+                    )
                 )
-            )
     elif "DJANGO_SETTINGS_MODULE" in os.environ:  # pragma: no cover
         sys.path.insert(0, os.path.abspath(os.getcwd()))
         try:
             # Django extension v2
             from django.conf import settings  # noqa
+            import dynaconf
+            import django
+
+            # see https://docs.djangoproject.com/en/4.2/ref/applications/
+            # at #troubleshooting
+            django.setup()
 
             settings.DYNACONF.configure()
         except AttributeError:
             settings = LazySettings()
 
-        if settings is not None:
-            _echo_enabled and click.echo(
+        if settings is not None and _echo_enabled:
+            click.echo(
                 click.style(
                     "Django app detected", fg="white", bg="bright_black"
                 )
@@ -256,7 +266,9 @@ def main(ctx, instance):
 @click.option("--django", default=os.environ.get("DJANGO_SETTINGS_MODULE"))
 @click.pass_context
 def init(ctx, fileformat, path, env, _vars, _secrets, wg, y, django):
-    """Inits a dynaconf project
+    """
+    Inits a dynaconf project.
+
     By default it creates a settings.toml and a .secrets.toml
     for [default|development|staging|testing|production|global] envs.
 
@@ -461,7 +473,11 @@ def get(key, default, env, unparse):
     if default is not empty:
         result = settings.get(key, default)
     else:
-        result = settings[key]  # let the keyerror raises
+        try:
+            result = settings[key]
+        except KeyError:
+            click.echo("Key not found", nl=False, err=True)
+            sys.exit(1)
 
     if unparse:
         result = unparse_conf_data(result)
@@ -513,8 +529,11 @@ def get(key, default, env, unparse):
     help="Output file is flat (do not include [env] name)",
 )
 def _list(env, key, more, loader, _all=False, output=None, flat=False):
-    """Lists all user defined config values
-    and if `--all` is passed it also shows dynaconf internal variables.
+    """
+    Lists user defined settings or all (including internal configs).
+
+    By default, shows only user defined. If `--all` is passed it also shows
+    dynaconf internal variables aswell.
     """
     if env:
         env = env.strip()
@@ -581,7 +600,7 @@ def _list(env, key, more, loader, _all=False, output=None, flat=False):
             value = empty
 
         if value is empty:
-            click.echo(click.style("Key not found", bg="red", fg="white"))
+            click.secho("Key not found", bg="red", fg="white", err=True)
             return
 
         click.echo(format_setting(key, value))
@@ -635,7 +654,7 @@ def _list(env, key, more, loader, _all=False, output=None, flat=False):
 )
 @click.option("-y", default=False, is_flag=True)
 def write(to, _vars, _secrets, path, env, y):
-    """Writes data to specific source"""
+    """Writes data to specific source."""
     _vars = split_vars(_vars)
     _secrets = split_vars(_secrets)
     loader = importlib.import_module(f"dynaconf.loaders.{to}_loader")
@@ -708,12 +727,14 @@ def write(to, _vars, _secrets, path, env, y):
     "--path", "-p", default=CWD, help="defaults to current directory"
 )
 def validate(path):  # pragma: no cover
-    """Validates Dynaconf settings based on rules defined in
-    dynaconf_validators.toml"""
+    """
+    Validates Dynaconf settings based on provided rules.
+
+    Rules should be defined in dynaconf_validators.toml
+    """
     # reads the 'dynaconf_validators.toml' from path
     # for each section register the validator for specific env
     # call validate
-
     path = Path(path)
 
     if not str(path).endswith(".toml"):
@@ -723,6 +744,7 @@ def validate(path):  # pragma: no cover
         click.echo(click.style(f"{path} not found", fg="white", bg="red"))
         sys.exit(1)
 
+    # parse validator file
     try:  # try tomlib first
         validation_data = tomllib.load(open(str(path), "rb"))
     except UnicodeDecodeError:  # fallback to legacy toml (TBR in 4.0.0)
@@ -733,6 +755,20 @@ def validate(path):  # pragma: no cover
         validation_data = toml.load(
             open(str(path), encoding=default_settings.ENCODING_FOR_DYNACONF),
         )
+    except tomllib.TOMLDecodeError as e:
+        click.echo(
+            click.style(
+                f"Error parsing TOML: {e}. Maybe it should be quoted.",
+                fg="white",
+                bg="red",
+            )
+        )
+        sys.exit(1)
+
+    # guarantee there is an environment
+    validation_data = {k.lower(): v for k, v in validation_data.items()}
+    if not validation_data.get("default"):
+        validation_data = {"default": validation_data}
 
     success = True
     for env, name_data in validation_data.items():
@@ -740,7 +776,8 @@ def validate(path):  # pragma: no cover
             if not isinstance(data, dict):  # pragma: no cover
                 click.echo(
                     click.style(
-                        f"Invalid rule for parameter '{name}'",
+                        f"Invalid rule for parameter '{name}'"
+                        "(this will be skipped)",
                         fg="white",
                         bg="yellow",
                     )
@@ -769,5 +806,53 @@ def validate(path):  # pragma: no cover
         sys.exit(1)
 
 
+from dynaconf.utils.inspect import (
+    KeyNotFoundError,
+    builtin_dumpers,
+    inspect_settings,
+    EnvNotFoundError,
+    OutputFormatError,
+)
+
+INSPECT_FORMATS = list(builtin_dumpers.keys())
+
+
+@main.command()
+@click.option("--key", "-k", help="Filters result by key.")
+@click.option("--env", "-e", help="Filters result by environment.", default="")
+@click.option(
+    "--format",
+    "-f",
+    help="The output format.",
+    default="json",
+    type=click.Choice(INSPECT_FORMATS),
+)
+@click.option(
+    "--descending",
+    "-d",
+    help="Set history loading order to 'last-first'",
+    default=False,
+    is_flag=True,
+)
+def inspect(key, env, format, descending):  # pragma: no cover
+    """
+    Inspect the loading history of the given settings instance.
+
+    Filters by key and environement, otherwise shows all.
+    """
+    try:
+        inspect_settings(
+            settings,
+            key_dotted_path=key,
+            env=env,
+            output_format=format,
+            ascending_order=(not descending),
+        )
+        click.echo()
+    except (KeyNotFoundError, EnvNotFoundError, OutputFormatError) as err:
+        click.echo(err)
+        sys.exit(1)
+
+
 if __name__ == "__main__":  # pragma: no cover
     main()
