@@ -1,36 +1,47 @@
 from __future__ import annotations
 
+from contextlib import suppress
 from os import environ
 
+from dynaconf.loaders.base import SourceMetadata
 from dynaconf.utils import missing
 from dynaconf.utils import upperfy
 from dynaconf.utils.parse_conf import parse_conf_data
 
 DOTENV_IMPORTED = False
-try:
+with suppress(ImportError, FileNotFoundError):
     from dynaconf.vendor.dotenv import cli as dotenv_cli
 
     DOTENV_IMPORTED = True
-except ImportError:
-    pass
-except FileNotFoundError:
-    pass
-
 
 IDENTIFIER = "env"
 
 
-def load(obj, env=None, silent=True, key=None):
+def load(obj, env=None, silent=True, key=None, validate=False):
     """Loads envvars with prefixes:
 
     `DYNACONF_` (default global) or `$(ENVVAR_PREFIX_FOR_DYNACONF)_`
     """
     global_prefix = obj.get("ENVVAR_PREFIX_FOR_DYNACONF")
     if global_prefix is False or global_prefix.upper() != "DYNACONF":
-        load_from_env(obj, "DYNACONF", key, silent, IDENTIFIER + "_global")
+        load_from_env(
+            obj,
+            "DYNACONF",
+            key,
+            silent,
+            IDENTIFIER + "_global",
+            validate=validate,
+        )
 
     # Load the global env if exists and overwrite everything
-    load_from_env(obj, global_prefix, key, silent, IDENTIFIER + "_global")
+    load_from_env(
+        obj,
+        global_prefix,
+        key,
+        silent,
+        IDENTIFIER + "_global",
+        validate=validate,
+    )
 
 
 def load_from_env(
@@ -40,6 +51,7 @@ def load_from_env(
     silent=False,
     identifier=IDENTIFIER,
     env=False,  # backwards compatibility bc renamed param
+    validate=False,
 ):
     if prefix is False and env is not False:
         prefix = env
@@ -52,13 +64,22 @@ def load_from_env(
         prefix = prefix.upper()
         env_ = f"{prefix}_"
 
+    # set source metadata
+    source_metadata = SourceMetadata(identifier, "unique", "global")
+
     # Load a single environment variable explicitly.
     if key:
         key = upperfy(key)
         value = environ.get(f"{env_}{key}")
         if value:
             try:  # obj is a Settings
-                obj.set(key, value, loader_identifier=identifier, tomlfy=True)
+                obj.set(
+                    key,
+                    value,
+                    loader_identifier=source_metadata,
+                    tomlfy=True,
+                    validate=validate,
+                )
             except AttributeError:  # obj is a dict
                 obj[key] = parse_conf_data(
                     value, tomlfy=True, box_settings=obj
@@ -69,6 +90,7 @@ def load_from_env(
         # Only known variables should be loaded from environment?
         ignore_unknown = obj.get("IGNORE_UNKNOWN_ENVVARS_FOR_DYNACONF")
 
+        # prepare data
         trim_len = len(env_)
         data = {
             key[trim_len:]: parse_conf_data(
@@ -88,12 +110,14 @@ def load_from_env(
             filter_strategy = obj.get("FILTER_STRATEGY")
             if filter_strategy:
                 data = filter_strategy(data)
-            obj.update(data, loader_identifier=identifier)
+            obj.update(
+                data, loader_identifier=source_metadata, validate=validate
+            )
 
 
 def write(settings_path, settings_data, **kwargs):
     """Write data to .env file"""
-    if not DOTENV_IMPORTED:
+    if not DOTENV_IMPORTED:  # pragma: no cover
         return
     for key, value in settings_data.items():
         quote_mode = (
